<?php

/**
 * Class EveryBoard_Page
 * TODO: Refactor and move into namespace
 */
class EveryBoard_Page
{
    /**
     * Initialize EveryBoard_Page class
     */
    public function __construct()
    {
        add_action('add_meta_boxes', [&$this, 'board_page_add_metaboxes']); // Adds custom meta boxes to edit page.
        add_action('save_post', [&$this, 'board_page_update_page']); // On page update execute action.
        add_filter('the_content',
            [&$this, 'board_page_content_override']); // Filter for overriding the_content if a board is chosen.

        // Previewing from Boards admin
        if (isset($_GET['board_preview']) && $_GET["board_preview"] === 'true') {
            add_action('wp_enqueue_scripts', [&$this, 'eb_page_enqueue']);
            add_filter('show_admin_bar', '__return_false');
            add_action('template_redirect', [&$this, 'cache_page']);
        }

        // Previewing from Pages admin
        add_filter('the_preview', [&$this, 'add_metadata_preview_filter']);
    }

    /**
     * @return void
     * @since 1.5.13
     */
    public function eb_page_enqueue()
    {
        wp_enqueue_script(
            'everyboard',
            EVERYBOARD_BASE . 'assets/dist/js/boardpreview.min.ugly.js',
            ['jquery'],
            EVERYBOARD_VERSION
        );

        $preview_id = $_GET['preview_id'] ?? '';

        wp_localize_script('everyboard', 'board', [
            'json' => EVERYBOARD_BASE . 'preview.php?preview=true',
            'json_data' => EVERYBOARD_BASE . 'preview.php?preview_json=true',
            'board_check' => EVERYBOARD_BASE . 'preview.php?get_board=true&id=' . $preview_id,
            'page_id' => $preview_id,
            'board_id' => $this->get_board_id($preview_id)
        ]);
    }

    /**
     * @return void
     */
    public function cache_page()
    {
        ob_start([&$this, 'ob_is_cached']);
    }

    /**
     * @param string $content
     *
     * @return string
     */
    public function ob_is_cached($content)
    {
        //TODO: Check if user has permission to view this
        if ( ! is_user_logged_in()) {
            wp_redirect(wp_login_url('https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']));
            die();
        }

        if (get_option('preview_data') !== null) {
            update_option('preview_data', $content);
        } else {
            add_option('preview_data', $content);
        }
        $urlBase = rtrim(EVERYBOARD_BASE, '/');

        return <<<HTML
<!DOCTYPE html>
<html>
    <head>
        <title>EveryBoard Preview</title>
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <style>.rotate {  }</style>
    </head>
    <body>
        <div id="wrapper">
            <iframe id="preview" src="$urlBase/preview.php?preview_data=true" width="100%" height="100%" frameborder="0"></iframe>
        </div>
        <script type="text/javascript">var board_url = "";</script>
        <script src="https://code.jquery.com/jquery-1.12.4.min.js"></script>
        <script type="text/javascript" src="$urlBase/assets/dist/js/boardpreview_action.min.ugly.js"></script>
    </body>
</html>
HTML;
    }

    /**
     * Adds custom meta boxes to edit board page in admin.
     *
     * @return void
     */
    public function board_page_add_metaboxes()
    {
        add_meta_box('page_board_side_meta', 'EveryBoard', [&$this, 'board_page_side_meta'], 'Page', 'side', 'core',
            null);
    }

    /**
     * Side meta box renders on edit page and displays options for connecting page to boards.
     *
     * @return void
     */
    public function board_page_side_meta()
    {
        // Get the active board id.
        $active_board_id = $this->get_active_board_id(get_the_ID());

        // Set up loop for boards cpt.
        $args = ['post_type' => 'everyboard', 'posts_per_page' => -1];
        $loop = new WP_Query($args);
        $selected = 'selected="selected"';

        // Print select board item.
        print '<p><strong>' . __('Choose an EveryBoard', 'everyboard') . '</strong></p>';
        print '<select name="everyboard_id" id="everyboard_id">';

        $board_title = '';
        print '<option value="">' . __('(No board)', 'everyboard') . '</option>';

        foreach ($loop->posts as $board_post) {
            print '<option value="' . $board_post->ID . '" ' . ($active_board_id === $board_post->ID ? $selected : '') . '>' . $board_post->post_title . '</option>';
            if ($active_board_id === $board_post->ID) {
                $board_title = $board_post->post_title;
            }
        }

        print '</select>';

        $link = get_edit_post_link($active_board_id);
        print " - <a href='" . $link . "' target='_blank'>Edit " . $board_title . '</a>';
    }

    /**
     * Executes on page update and saves custom field data.
     *
     * @return void
     * @since 1.5.13
     */
    public function board_page_update_page()
    {
        $target_post_id = $_REQUEST['post_ID'] ?? null;

        // Do not update meta when we're only previewing.
        // Instead, try to direct the updating to the autosave that should exist now.
        $is_previewing = ($_REQUEST['wp-preview'] ?? '') === 'dopreview';
        if ($is_previewing) {
            $preview = wp_get_post_autosave($target_post_id);
            if ($preview instanceof WP_Post) {
                $target_post_id = $preview->ID;
            } else {
                return;
            }
        }

        if (isset($target_post_id, $_REQUEST['post_type'], $_REQUEST['everyboard_id']) && $_REQUEST['post_type'] === 'page') {
            update_metadata( 'post', $target_post_id, 'everyboard_id', $_REQUEST['everyboard_id'] );
        }
    }

    /**
     * Override the_content and return Board Data if a board is active.
     *
     * @param string $content
     *
     * @return string
     * @since 1.5.13
     */
    public function board_page_content_override($content)
    {
        // If this is not a single page, return the_content as is.
        if ( ! is_singular('page')) {
            return $content;
        }

        $board = $this->get_page_board();

        // If board has been removed or trashed
        if (null === $board || $board->post_status === 'trash') {
            return $content;
        }

        $board_renderer = new EveryBoard_Renderer();

        return $board_renderer->render($this->get_board_json($board->ID));
    }

    /**
     * Add the autosave meta to get_post_metadata for preview meta data.
     *
     * @param WP_Post $post Post object.
     *
     * @return WP_Post
     * @since 2.4.2
     */
    public function add_metadata_preview_filter(WP_Post $post): WP_Post
    {
       add_filter('get_post_metadata', [&$this, 'get_metadata_from_autosave'], 10, 4);
       return $post;
    }

    /**
     * Filters post meta retrieval to get everyboard_id metadata from the actual autosave post,
     * and not its parent.
     *
     * @param mixed  $value     Meta value to filter.
     * @param int    $object_id Object ID.
     * @param string $meta_key  Meta key to filter a value for.
     * @param bool   $single    Whether to return a single value. Default false.
     *
     * @return mixed Original meta value if it's another key, the object doesn't exist,
     *               the post type is a revision or the post ID doesn't match the object ID.
     *               Otherwise, the revisioned meta value is returned.
     * @since 2.4.2
     */
    public function get_metadata_from_autosave($value, int $object_id, string $meta_key, bool $single)
    {
        if ($meta_key !== 'everyboard_id') {
            return $value;
        }

        $post = get_post();
        if (empty($post) || $post->ID !== $object_id || $post->post_type === 'revision') {
            return $value;
        }

        // Grab the autosave.
        $preview = wp_get_post_autosave($post->ID);
        if (! $preview instanceof WP_Post) {
            return $value;
        }

        // Return the autosave revisioned meta.
        return get_post_meta($preview->ID, $meta_key, $single);
    }

    /**
     * Retrieve board json from id
     *
     * @param int $board_id
     *
     * @return array|null
     * @since 1.5.13
     */
    protected function get_board_json($board_id)
    {
        if (isset($_GET['board_preview']) && $_GET['board_preview'] === 'true') {
            return get_post_meta($board_id, 'everyboard_json_temp');
        }

        return get_post_meta($board_id, 'everyboard_json');
    }

    /**
     * Retrieve id of board connected to current page
     *
     * @param int|false $post_id
     *
     * @return int
     * @since 1.5.13
     */
    protected function get_board_id($post_id = false)
    {
        if (isset($_GET['board_id'])) {
            return (int)$_GET['board_id'];
        }

        if ($post_id === false || empty($post_id)) {
            $post_id = get_the_ID();
        }

        $board_id = get_post_meta($post_id, 'everyboard_id');

        return count($board_id) > 0 ? (int)$board_id[0] : 0;
    }

    /**
     * Retrieve board connected to current page
     *
     * @return null|WP_Post
     * @since 1.5.13
     */
    protected function get_page_board()
    {
        $board_id = $this->get_board_id();
        // If board is chosen
        if ($board_id !== null && $board_id !== 0) {
            return get_post($board_id);
        }

        return null;
    }

    protected function get_active_board_id(int $page_id): int
    {
        $active_board_settings = (array)get_post_custom_values('everyboard_id', $page_id);

        return ! empty($active_board_settings) ? (int)$active_board_settings[0] : 0;
    }
}
